'''
Errors

This example shows how errors are automatically given proper styling,
thanks to uswds_forms.UswdsForm and Django 1.11's new form rendering API.
'''

from django.shortcuts import render
from django import forms
import uswds_forms


class MyForm(uswds_forms.UswdsForm):
    text = forms.CharField(label="Text input label")

    checkbox = forms.BooleanField(label='I agree to the terms of service')

    def clean(self):
        super().clean()
        self.add_error('text', 'Helpful error message #1')
        self.add_error('text', 'Helpful error message #2')
        self.add_error(None, 'Helpful non-field error message #1')
        self.add_error(None, 'Helpful non-field error message #2')

def view(request):
    # Since this example is specifically made to show errors, we'll
    # "simulate" a POST request here.

    form = MyForm({'text': 'blah'})
    form.is_valid()
    return render(request, 'examples/errors.html', {'form': form})



from ..views import EXAMPLES


class TestEachExample(type):
    '''
    This is a (hopefully) simple metaclass that makes it possible
    to create a TestCase that runs a generic test for every
    Example in the example gallery.

    The class this metaclass is applied to must have one method
    with the following signature:

        def test(self, example):
            ...

    This single method will be used as a "template" from which
    multiple tests are created--one for every Example in the
    gallery. Each test will be called `test_XXX`, where `XXX`
    is the name of the test.
    '''

    @classmethod
    def make_test_method_for_example(cls, test, example):
        def wrapper(self):
            test(self, example)

        wrapper.__name__ = 'test_' + example.basename
        return wrapper

    def __new__(cls, name, bases, namespace):
        if 'test' not in namespace:
            raise Exception('Subclasses must implement a "test"'
                            'method that takes an Example instance.')

        test = namespace.pop('test')

        for example in EXAMPLES.values():
            test_method = cls.make_test_method_for_example(test, example)
            namespace[test_method.__name__] = test_method

        return type.__new__(cls, name, bases, namespace)



import abc
from unittest import SkipTest
from typing import Dict
from django.test import TestCase

from .util import TestEachExample


class ExamplesReturn200Tests(TestCase, metaclass=TestEachExample):
    def test(self, example):
        res = self.client.get('/example/' + example.basename)
        self.assertEqual(res.status_code, 200)


class ExampleMixin(metaclass=abc.ABCMeta):
    @property
    @abc.abstractmethod
    def url(self) -> str:
        '''
        URL for example. Must be defined by subclasses.
        '''

    @property
    @abc.abstractmethod
    def valid_post(self) -> Dict[str, str]:
        '''
        Valid POST data for example to succeed. Must be defined by subclasses.
        If the form has no required fields, set this to {}.
        '''

    def test_empty_post_shows_errors(self):
        if self.valid_post == {}:
            raise SkipTest('Form has no required fields')
        res = self.client.post(self.url, {})
        self.assertContains(res, 'Submission unsuccessful')

    def test_valid_post_shows_success(self):
        res = self.client.post(self.url, self.valid_post)
        self.assertContains(res, 'Submission successful')


class RadiosExampleTests(ExampleMixin, TestCase):
    url = '/example/radios'

    valid_post = {
        'president': 'washington',
    }


class CheckboxesExampleTests(ExampleMixin, TestCase):
    url = '/example/checkboxes'

    valid_post = {}  # type: Dict[str, str]


class DateExampleTests(ExampleMixin, TestCase):
    url = '/example/date'

    valid_post = {
        'date_1': '4',
        'date_2': '28',
        'date_0': '2016',
    }


class EverythingExampleTests(ExampleMixin, TestCase):
    url = '/example/everything'

    valid_post = {
        'president': 'washington',
        'park': 'foo',
        'date_1': '4',
        'date_2': '28',
        'date_0': '2016',
    }

    def test_non_field_errors_are_displayed(self):
        res = self.client.post(self.url, {
            'trigger_non_field_error': 'on',
        })
        self.assertContains(res, 'This is the non-field error you requested')



from datetime import date
from collections import namedtuple

from django.core.exceptions import ValidationError
from django.forms import MultiWidget, NumberInput
from django.forms.fields import MultiValueField, IntegerField


__all__ = (
    'UswdsDateField',
    'UswdsDateWidget',
)


# This is just a convenience that allows us to reference the
# fields without having to remember what index they are in the
# tuple ordering.
DateTuple = namedtuple('DateTuple', ['year', 'month', 'day'])


FIELD_ORDERING = DateTuple._fields
YEAR_ID = FIELD_ORDERING.index('year')
MONTH_ID = FIELD_ORDERING.index('month')
DAY_ID = FIELD_ORDERING.index('day')


class UswdsDateWidget(MultiWidget):
    '''
    A :class:`django.forms.MultiWidget` for a USWDS-style date, with
    separate number fields for date, month, and year.

    This widget is used automatically by
    :class:`uswds_forms.UswdsDateField`, so you probably won't need
    to use it directly. However, it can be subclassed in case you need
    to customize it.
    '''

    #: This is the default template used by the widget, which can
    #: be overridden if needed.
    template_name = 'uswds_forms/date.html'

    year_attrs = {
        'pattern': r'[0-9]{4}',
        'min': '1900',
        'max': '9999',
    }

    month_attrs = {
        'pattern': r'0?[1-9]|1[012]',
        'min': '1',
        'max': '12',
    }

    day_attrs = {
        'pattern': r'0?[1-9]|1[0-9]|2[0-9]|3[01]',
        'min': '1',
        'max': '31',
    }

    def __init__(self, attrs=None):
        widgets = DateTuple(
            year=NumberInput(attrs=self.year_attrs),
            month=NumberInput(attrs=self.month_attrs),
            day=NumberInput(attrs=self.day_attrs),
        )
        super().__init__(widgets, attrs=attrs)

    def decompress(self, value):
        if value:
            return list(DateTuple(
                year=value.year,
                month=value.month,
                day=value.day
            ))
        return [None, None, None]

    @staticmethod
    def get_field_names(name):
        # Note that this is actually dependent on the way our superclass
        # names our subwidgets. The naming scheme should be pretty stable,
        # though.
        return DateTuple(
            year=name + '_{}'.format(YEAR_ID),
            month=name + '_{}'.format(MONTH_ID),
            day=name + '_{}'.format(DAY_ID)
        )

    def get_context(self, name, value, attrs):
        '''
        Returns the context for the widget's template. This returns
        a superset of what's provided by its superclass'
        :meth:`~django.forms.MultiWidget.get_context`
        implementation, adding the following keys to ``widget``:

        * ``'hint_id'``: The unique id of some hint text showing users
          an example of what they can enter.

        * ``'subwidgets'``: This has the same iterable value described
          in the superclass documentation, but it has been enhanced
          such that its ``year``, ``month``, and ``day`` properties are
          aliases to its entries. Using these aliases can potentially make
          templates more readable.
        '''

        ctx = super().get_context(name, value, attrs)
        widget = ctx['widget']
        hint_id = '%s_%s' % (widget['attrs']['id'], 'hint')
        for subwidget in widget['subwidgets']:
            subwidget['attrs'].update({
                'class': 'usa-input-inline',
                'aria-describedby': hint_id,
            })
        widget['subwidgets'] = DateTuple(*widget['subwidgets'])
        widget.update({'hint_id': hint_id})
        return ctx


class UswdsDateField(MultiValueField):
    '''
    A :class:`django.forms.MultiValueField` for a USWDS-style date.
    Its value normalizes to a Python :class:`datetime.date` object.

    For an example of how this looks in practice, see the
    `USWDS date input example
    <https://standards.usa.gov/components/form-controls/#date-input>`_.
    '''

    widget = UswdsDateWidget

    def __init__(self, *args, **kwargs):
        fields = DateTuple(
            year=IntegerField(),
            month=IntegerField(),
            day=IntegerField(),
        )
        super().__init__(fields, *args, **kwargs)

    def compress(self, data_list):
        if data_list:
            fields = DateTuple(*data_list)
            try:
                return date(
                    year=fields.year,
                    month=fields.month,
                    day=fields.day
                )
            except ValueError as e:
                raise ValidationError('Invalid date: %s.' % str(e))
        return None



'''
Date

This example shows how to render a date input using a
uswds_forms.UswdsDateField.
'''

from django.shortcuts import render
import uswds_forms


class MyForm(uswds_forms.UswdsForm):
    date = uswds_forms.UswdsDateField(label="What is your favorite date?")


def view(request):
    return render(request, 'examples/date.html', {
        'form': MyForm() if request.method == 'GET' else MyForm(request.POST)
    })



from django import forms


__all__ = (
    'UswdsRadioSelect',
    'UswdsCheckboxSelectMultiple',
    'UswdsMultipleChoiceField',
)


class UswdsWidgetMixin:
    option_template_name = 'uswds_forms/input_option.html'

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)  # type: ignore
        widget_attrs = context['widget']['attrs']
        if 'class' not in widget_attrs:
            widget_attrs['class'] = 'usa-unstyled-list'
        if 'id' not in widget_attrs:
            raise ValueError('USWDS-style inputs must have "id" '
                             'attributes')
        return context


class UswdsRadioSelect(UswdsWidgetMixin, forms.widgets.RadioSelect):
    '''
    This subclass of :class:`django.forms.RadioSelect` styles
    radio buttons appropriately for USWDS.

    You can use this in a :class:`django.forms.ChoiceField` to get a list
    of radio buttons instead of a ``<select>`` element for your
    choices.

    For an example of how this looks in practice, see the
    `USWDS radio buttons example
    <https://standards.usa.gov/components/form-controls/#radio-buttons>`_.
    '''


class UswdsCheckboxSelectMultiple(UswdsWidgetMixin,
                                  forms.widgets.CheckboxSelectMultiple):
    '''
    This subclass of :class:`django.forms.CheckboxSelectMultiple` styles
    grouped checkboxes appropriately for USWDS.

    You can use this in a :class:`django.forms.MultipleChoiceField` to
    get a list of checkboxes instead of a ``<select multiple>`` element
    for your choices.

    For an example of how this looks in practice, see the
    `USWDS checkboxes example
    <https://standards.usa.gov/components/form-controls/#checkboxes>`_.
    '''


class UswdsMultipleChoiceField(forms.fields.MultipleChoiceField):
    '''
    This is just a :class:`django.forms.MultipleChoiceField`
    that uses the :class:`~uswds_forms.UswdsCheckboxSelectMultiple` widget.

    We've provided this field for convenience because
    the usability of the default :class:`django.forms.SelectMultiple`
    is so terrible that you'll probably never want to use it. `Burn
    your select tags! <https://www.youtube.com/watch?v=CUkMCQR4TpY>`_
    '''

    widget = UswdsCheckboxSelectMultiple



'''
TODO: Put human-readable name for '_startexample_template' here.

TODO: Put example description here.
'''

from django.shortcuts import render
from django import forms
import uswds_forms


class MyForm(uswds_forms.UswdsForm):
    president = forms.ChoiceField(
        label="Example form field",
        widget=uswds_forms.UswdsRadioSelect,
        choices=(
            ('a', 'Choice A'),
            ('b', 'Choice B'),
        )
    )


def view(request):
    return render(request, 'examples/_startexample_template.html', {
        'form': MyForm() if request.method == 'GET' else MyForm(request.POST)
    })



'''
    This command-line program makes it easy to get the Sphinx
    inventory file for a third-party project, and catalog it.

    Run this script without any arguments for help.
'''

import sys
import subprocess

import conf


if __name__ == '__main__':
    mapping = conf.intersphinx_mapping
    if len(sys.argv) < 2 or sys.argv[1] not in mapping:
        print("Usage: {} [{}]".format(
            sys.argv[0],
            '|'.join(mapping.keys()),
        ))
        sys.exit(1)

    uri, obj_uri = mapping[sys.argv[1]]
    if obj_uri is None:
        obj_uri = uri + '/objects.inv'

    # http://stackoverflow.com/a/36831198
    subprocess.check_call([
        sys.executable, '-m', 'sphinx.ext.intersphinx',
        obj_uri,
    ])



import pprint
from pathlib import Path
from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from selenium import webdriver

from .util import TestEachExample

AXE_JS = Path(__file__).resolve().parent / 'vendor' / 'axe.js'


class AxeTests(StaticLiveServerTestCase, metaclass=TestEachExample):
    '''
    Run aXe-core on every example's URL. Raise an exception if
    there are any accessibility issues and log any violations to stdout.
    '''

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.axe_js = AXE_JS.read_text()
        cls.selenium = webdriver.PhantomJS()
        cls.selenium.implicitly_wait(10)

    @classmethod
    def tearDownClass(cls):
        cls.selenium.quit()
        super().tearDownClass()

    def test(self, example):
        self.selenium.get('%s%s' % (self.live_server_url, example.url))

        result = self.selenium.execute_async_script(
            self.axe_js +
            r'''
            var callback = arguments[arguments.length - 1];
            axe.run(function(err, results) {
              callback({
                error: err && err.toString(),
                results: results
              });
            });
            ''')

        error = result['error']
        if error is not None:
            raise Exception('axe.run() failed: {}'.format(error))

        violations = [v for v in result['results']['violations']]
        if violations:
            pprint.pprint(violations)
            print("For more details, please install the aXe browser "
                  "extension.")
            print("Learn more at: https://www.deque.com/products/axe/")

            raise Exception('axe.run() found violations: ' +
                            ', '.join(v['id'] for v in violations))



'''
The kitchen sink

This example combines almost every feature of django-uswds-forms
into one big form.
'''

from django.shortcuts import render
from django import forms
import uswds_forms


class EverythingForm(uswds_forms.UswdsForm):
    required_css_class = 'usa-input-required'

    president = forms.ChoiceField(
        label="Who is your favorite president?",
        widget=uswds_forms.UswdsRadioSelect,
        help_text=("If you don't see your favorite, just pick your "
                   "favorite of the ones we've listed."),
        choices=(
            ('washington', 'George Washington'),
            ('adams', 'John Adams'),
            ('jefferson', 'Thomas Jefferson'),
        )
    )

    park = forms.CharField(
        label=("If you could choose the name of the next national park, "
               "what would it be?"),
        help_text='Note that "Parky McParkface" is not a valid response.'
    )

    states = uswds_forms.UswdsMultipleChoiceField(
        label="What states have you visited?",
        required=False,
        choices=(
            ('OH', 'Ohio'),
            ('IL', 'Illinois'),
            ('CA', 'California'),
        )
    )

    date = uswds_forms.UswdsDateField(
        label="What is your favorite date?"
    )

    trigger_non_field_error = forms.BooleanField(
        label=("After submitting this form, trigger a "
               "non-field error."),
        required=False,
    )


def view(request):
    if request.method == 'POST':
        form = EverythingForm(request.POST)
        if form.data.get('trigger_non_field_error'):
            form.add_error(None, 'This is the non-field error you requested.')
    else:
        form = EverythingForm()

    return render(request, 'examples/everything.html', {
        'form': form
    })



from unittest.mock import patch
from django.test import TestCase, override_settings, RequestFactory
from django.shortcuts import render
from django.forms.renderers import Jinja2, DjangoTemplates
from django.conf import settings

from .util import TestEachExample


@override_settings(
    TEMPLATES=[settings.DJANGO_TEMPLATE_BACKEND,
               settings.JINJA2_TEMPLATE_BACKEND]
)
class TemplateEngineParityTests(TestCase, metaclass=TestEachExample):
    '''
    This renders each example using its Django and Jinja2 template
    and ensures that both produce the same HTML.
    '''

    def make_renderer(self, engine_name, form_renderer):
        def force_render_using_engine(req, template_name, ctx):
            ctx['form'].renderer = form_renderer
            return render(req, template_name, ctx, using=engine_name)
        return force_render_using_engine

    def render_example(self, request, example, renderer):
        with patch.object(example.module, 'render', renderer):
            content = example.render(request)
        return '\n'.join([
            line for line in content.split('\n')
            if 'csrfmiddlewaretoken' not in line
        ])

    def test(self, example):
        self.maxDiff = 5000
        factory = RequestFactory()
        django = self.render_example(
            factory.get('/'),
            example,
            self.make_renderer('django', DjangoTemplates())
        )

        jinja2 = self.render_example(
            factory.get('/'),
            example,
            self.make_renderer('jinja2', Jinja2())
        )

        self.assertHTMLEqual(
            django,
            jinja2,
            'renders of "{}" example must match'.format(example.basename)
        )



import re
from pathlib import Path
from argparse import ArgumentParser
from django.core.management.base import BaseCommand, CommandError

from app.example import Example, APP_DIR

SLUG_RE = re.compile('^[A-Za-z0-9_]+$')
SLUG_HELP = "Alphanumerics and underscores only"
EXAMPLE_NAMES_PATH = "app.views.EXAMPLE_NAMES"
EXAMPLE_TESTS_PATH = APP_DIR / 'tests' / 'test_examples.py'
TEMPLATE_NAME = '_startexample_template'


def untemplatize(f: Path, example: Example):
    return f.read_text().replace(TEMPLATE_NAME, example.basename)


class Command(BaseCommand):
    help = 'Creates a new example for the gallery.'

    def add_arguments(self, parser: ArgumentParser):
        parser.add_argument(
            'example_slug',
            help="Slug for example. {}.".format(SLUG_HELP)
        )
        parser.add_argument(
            '--undo',
            action='store_true',
            help='Undo an earlier invocation of this command.'
        )

    def undo_copy(self, src: Path, dest: Path, example: Example) -> None:
        relpath = dest.relative_to(Path.cwd())
        if not dest.exists():
            self.stdout.write("Hmm, {} does not exist.".format(relpath))
        elif dest.read_text() != untemplatize(src, example):
            self.stdout.write("{} has changed, not deleting.".format(relpath))
        else:
            dest.unlink()
            self.stdout.write("Deleted {}.".format(relpath))

    def copy(self, src: Path, dest: Path, example: Example) -> None:
        relpath = dest.relative_to(Path.cwd())
        if dest.exists():
            self.stdout.write("{} exists, not overwriting.".format(relpath))
        else:
            dest.write_text(untemplatize(src, example))
            self.stdout.write("Created {}.".format(relpath))

    def handle(self, example_slug: str, undo: bool, **kwargs):
        if not SLUG_RE.match(example_slug):
            raise CommandError('Invalid slug! {}.'.format(SLUG_HELP))

        template = Example(TEMPLATE_NAME)
        ex = Example(example_slug)

        if undo:
            self.undo_copy(template.template_path, ex.template_path, ex)
            self.undo_copy(template.jinja2_path, ex.jinja2_path, ex)
            self.undo_copy(template.python_path, ex.python_path, ex)
        else:
            self.copy(template.template_path, ex.template_path, ex)
            self.copy(template.jinja2_path, ex.jinja2_path, ex)
            self.copy(template.python_path, ex.python_path, ex)

            self.stdout.write("\nDone! Now edit the above files.")
            self.stdout.write("Then, add '{}' to {}.".format(
                example_slug,
                EXAMPLE_NAMES_PATH,
            ))
            self.stdout.write(
                "You may also want to write tests for "
                "the example in {}.".format(
                    EXAMPLE_TESTS_PATH.relative_to(Path.cwd())
                )
            )



"""
Django settings for example project.

Generated by 'django-admin startproject' using Django 1.8.

For more information on this file, see
https://docs.djangoproject.com/en/1.8/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.8/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
import sys
from django.core.exceptions import ImproperlyConfigured

try:
    import whitenoise
except ImportError:
    whitenoise = None

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.8/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'this is a demo app'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['*']


# Application definition

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'uswds_forms',
    'app',
)

MIDDLEWARE_CLASSES = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
]

if whitenoise is not None:
    # This is only needed for deploying on heroku/cloudfoundry/etc.
    MIDDLEWARE_CLASSES.insert(
        0,
        'whitenoise.middleware.WhiteNoiseMiddleware'
    )

ROOT_URLCONF = 'example.urls'

DJANGO_TEMPLATE_BACKEND = {
    'BACKEND': 'django.template.backends.django.DjangoTemplates',
    'DIRS': [],
    'APP_DIRS': True,
    'OPTIONS': {
        'context_processors': [
            'django.template.context_processors.debug',
            'django.template.context_processors.request',
            'django.contrib.auth.context_processors.auth',
            'django.contrib.messages.context_processors.messages',
        ],
    },
}

JINJA2_TEMPLATE_BACKEND = {
    'BACKEND': 'django.template.backends.jinja2.Jinja2',
    'DIRS': [],
    'APP_DIRS': True,
    'OPTIONS': {
        'environment': 'app.jinja2.environment',
    }
}

template_engine = os.environ.get('TEMPLATE_ENGINE', 'django')

if template_engine == 'jinja2':
    FORM_RENDERER = 'django.forms.renderers.Jinja2'
    TEMPLATES = [JINJA2_TEMPLATE_BACKEND, DJANGO_TEMPLATE_BACKEND]
elif template_engine == 'django':
    FORM_RENDERER = 'django.forms.renderers.DjangoTemplates'
    TEMPLATES = [DJANGO_TEMPLATE_BACKEND]
else:
    raise ImproperlyConfigured('Invalid TEMPLATE_ENGINE: ' + template_engine)

if len(sys.argv) > 1 and sys.argv[1] == 'runserver':
    print("Examples will be rendered using "
          "{} templates.".format(template_engine))

WSGI_APPLICATION = 'example.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.8/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}


# Internationalization
# https://docs.djangoproject.com/en/1.8/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.8/howto/static-files/

STATIC_URL = '/static/'

STATIC_ROOT = os.path.join(BASE_DIR, 'static')

DOCS_URL = "http://django-uswds-forms.readthedocs.io/en/latest/"



from django import forms
from django.utils.safestring import SafeString

from .fieldset_helper import fieldset
from .errors import UswdsErrorList


__all__ = (
    'UswdsForm',
)


class UswdsForm(forms.Form):
    '''
    This is a subclass of :class:`django.forms.Form` that provides
    some functionality for rendering USWDS forms. Its constructor
    takes the exact same arguments, but some defaults are
    changed:

    * ``error_class`` defaults to :class:`uswds_forms.UswdsErrorList`,
      so that errors are formatted nicely.

    * :py:attr:`~django.forms.Form.label_suffix` defaults to the
      empty string, since none of the USWDS example forms have
      colons after the label names.
    '''

    def __init__(self, *args, **kwargs):
        if 'error_class' not in kwargs:
            kwargs['error_class'] = UswdsErrorList
        kwargs.setdefault('label_suffix', '')
        super().__init__(*args, **kwargs)

    def as_fieldsets(self):
        '''
        Like other convenience methods such as
        :py:meth:`~django.forms.Form.as_p` and
        :py:meth:`~django.forms.Form.as_table`, this method renders all
        the form's fields as a series of ``<fieldset>`` elements.

        Under the hood, this just iterates over the form's fields and
        renders them via the
        :ref:`fieldset template tag <fieldset-template-tag>`, so you
        can use that if you need more granular control over rendering.
        '''

        return SafeString('\n'.join([fieldset(field) for field in self]))



'''
Multiple checkboxes

This example shows how to render groups of checkboxes using
a uswds_forms.UswdsMultipleChoiceField.
'''

from django.shortcuts import render
import uswds_forms


class MyForm(uswds_forms.UswdsForm):
    states = uswds_forms.UswdsMultipleChoiceField(
        label="What states have you visited?",
        required=False,
        choices=(
            ('OH', 'Ohio'),
            ('IL', 'Illinois'),
            ('CA', 'California'),
        )
    )


def view(request):
    return render(request, 'examples/checkboxes.html', {
        'form': MyForm() if request.method == 'GET' else MyForm(request.POST)
    })



'''
Radio buttons

This example shows how to render radio buttons using the
uswds_forms.UswdsRadioSelect widget.
'''

from django.shortcuts import render
from django import forms
import uswds_forms


class MyForm(uswds_forms.UswdsForm):
    president = forms.ChoiceField(
        label="Who is your favorite president?",
        widget=uswds_forms.UswdsRadioSelect,
        help_text=("If you don't see your favorite, just pick your "
                   "favorite of the ones we've listed."),
        choices=(
            ('washington', 'George Washington'),
            ('adams', 'John Adams'),
            ('jefferson', 'Thomas Jefferson'),
        )
    )


def view(request):
    return render(request, 'examples/radios.html', {
        'form': MyForm() if request.method == 'GET' else MyForm(request.POST)
    })



from django.test import TestCase
from ..render_source import clean_python_source, clean_template_source


class RenderSourceTests(TestCase):
    def test_clean_python_source_works(self):
        self.assertEqual(
            clean_python_source("'''boop\n'''\nhello()\nthere()\n"),
            "hello()\nthere()"
        )

    def test_clean_template_source_works(self):
        self.assertEqual(
            clean_template_source(r"{% include 'foo' %}" + "\nblah\n"),
            'blah'
        )



from django.core.management import call_command
from django.core.management.base import CommandError
from django.test import TestCase
from django.utils.six import StringIO
from django.utils.module_loading import import_string

from app.example import Example
from app.management.commands.startexample import (
    EXAMPLE_NAMES_PATH,
    EXAMPLE_TESTS_PATH,
)


class ClosepollTest(TestCase):
    def setUp(self):
        self.example = Example('zzz')
        self.paths = [
            self.example.template_path,
            self.example.jinja2_path,
            self.example.python_path
        ]
        self.cleanup_paths()

    def tearDown(self):
        self.cleanup_paths()

    def cleanup_paths(self):
        for path in self.paths:
            if path.exists():
                path.unlink()

    def call(self, *args):
        out = StringIO()
        call_command('startexample', *args, stdout=out)
        return out.getvalue()

    def test_example_names_path_exists(self):
        import_string(EXAMPLE_NAMES_PATH)

    def test_example_tests_path_exists(self):
        self.assertTrue(EXAMPLE_TESTS_PATH.exists())

    def test_undo_does_not_delete_modified_files(self):
        self.call(self.example.basename)
        for path in self.paths:
            path.write_text("i am changed!")
        output = self.call(self.example.basename, '--undo')
        self.assertIn('has changed, not deleting', output)

    def test_undo_does_not_explode_if_files_do_not_exist(self):
        output = self.call(self.example.basename, '--undo')
        self.assertIn('does not exist', output)

    def test_create_does_not_overwrite(self):
        self.call(self.example.basename)
        output = self.call(self.example.basename)
        self.assertIn('not overwriting', output)

    def test_create_works(self):
        output = self.call(self.example.basename)
        self.assertIn('Created', output)
        for path in self.paths:
            self.assertTrue(path.exists(), '{} was created'.format(path))
        self.assertIn(self.example.basename,
                      self.example.python_path.read_text())

    def test_undo_works(self):
        self.call(self.example.basename)
        output = self.call(self.example.basename, '--undo')
        self.assertIn('Deleted', output)
        for path in self.paths:
            self.assertFalse(path.exists(), '{} was removed'.format(path))

    def test_invalid_slug_raises_error(self):
        with self.assertRaisesRegex(CommandError, 'Invalid slug'):
            self.call('boop?!')



from django.conf import settings
from django.http import HttpResponseNotFound
from django.shortcuts import render
from collections import OrderedDict

from .example import Example

# This is a list of every example in the gallery; the order determines
# the order in which they are listed on the site.
EXAMPLE_NAMES = [
    'radios',
    'checkboxes',
    'date',
    'errors',
    'everything',
]

EXAMPLES = OrderedDict([
    (name, Example(name, load=True)) for name in EXAMPLE_NAMES
])


def ctx(**kwargs):
    return {
        'EXAMPLES': EXAMPLES,
        'DOCS_URL': settings.DOCS_URL,
        **kwargs
    }


def example(request, name):
    example = EXAMPLES.get(name)

    if example is None:
        return HttpResponseNotFound('Example not found.')

    return render(request, 'example.html', ctx(
        rendered_example=example.render(request),
        example=example,
    ))


def home(request):
    return render(request, 'home.html', ctx())



from django import forms
from django.forms.utils import flatatt
from django.utils.html import format_html
from django.utils.safestring import SafeString
from django.template.loader import render_to_string

from . import date


__all__ = (
    'fieldset',
)

# The template path for a <fieldset> wrapping a form input. It
# should exist for all supported template engines.
TEMPLATE_NAME = 'uswds_forms/fieldset.html'

# These are widget classes that consist of multiple sub-widgets. We'd
# like to use a <legend> element with these, instead of a <label>, so
# that screen-readers contextualize them properly.
LEGEND_WIDGETS = (
    date.UswdsDateWidget,
    forms.CheckboxSelectMultiple,
    forms.RadioSelect
)


def get_context(field):
    '''
    Get the context for the fieldset template, regardless of template
    engine backend, and return it.
    '''

    use_legend = isinstance(field.field.widget, LEGEND_WIDGETS)

    label_attrs = {}

    if field.errors:
        label_attrs['class'] = 'usa-input-error-label'

    if use_legend:
        aria_hidden_label_tag = create_aria_hidden_label_tag(
            field,
            label_attrs
        )
    else:
        aria_hidden_label_tag = None

    label_tag = field.label_tag(attrs=label_attrs)

    return {
        'field': field,
        'put_field_before_label': isinstance(field.field.widget,
                                             forms.CheckboxInput),
        'use_legend': use_legend,
        'aria_hidden_label_tag': aria_hidden_label_tag,
        'label_tag': label_tag,
    }


def fieldset(field):
    '''
    Render the given bound field as a ``<fieldset>``. The field’s label
    is also rendered, along with any associated form errors and help text.

    .. highlight:: html+django

    Once added to a Jinja2 environment, this function can be used
    from Jinja2 templates like so::

        {{ fieldset(my_form.name) }}
        {{ fieldset(my_form.address) }}
        {{ fieldset(my_form.birthday) }}
    '''

    return SafeString(render_to_string(TEMPLATE_NAME, get_context(field)))


def create_aria_hidden_label_tag(field, attrs):
    '''
    A <label> for sighted users only.

    Sighted users will see this instead of <legend> for groups of
    sub-widgets because it's easier to visually style.

    It can't include a 'for' attribute because the attribute will
    likely be duplicated in the sub-widget grouping, which can
    confuse browsers because multiple labels for the same id will
    exist.

    Other than that, much of this code is taken from Django's
    implementation of BoundField.label_tag().
    '''

    attrs = {'aria_hidden': 'true', **attrs}
    if field.field.required and hasattr(field.form, 'required_css_class'):
        if 'class' in attrs:
            attrs['class'] += ' ' + field.form.required_css_class
        else:
            attrs['class'] = field.form.required_css_class
    attrs = flatatt(attrs)
    return SafeString(format_html('<label{}>{}</label>', attrs, field.label))



import ast
from pathlib import Path
from django.utils.safestring import SafeString

try:
    from pygments import highlight
    from pygments.lexers import PythonLexer
    from pygments.lexers.templates import HtmlDjangoLexer
    from pygments.formatters import HtmlFormatter
except ImportError:
    highlight = None


def render_source(contents: str, filetype: str):
    if highlight is None:
        return contents
    else:
        formatter = HtmlFormatter(noclasses=True, style='trac')
        if filetype == 'html+django':
            lexer = HtmlDjangoLexer()
        elif filetype == 'python':
            lexer = PythonLexer()
        else:
            raise ValueError('unknown filetype: {}'.format(filetype))
        return SafeString(highlight(contents, lexer, formatter))


def clean_python_source(source: str):
    '''
    Remove the leading docstring from the given source code.
    '''

    mod = ast.parse(source)
    first_non_docstring = mod.body[1]
    return '\n'.join(source.splitlines()[first_non_docstring.lineno - 1:])


def clean_template_source(source: str):
    '''
    Remove any un-indented {% include %} tags in the given template source.
    '''

    return '\n'.join(
        line for line in source.splitlines()
        if not line.startswith(r'{% include')
    )


def render_template_source(f: Path):
    return render_source(clean_template_source(f.read_text()), 'html+django')


def render_python_source(f: Path):
    return render_source(clean_python_source(f.read_text()), 'python')



from django import template

from ..fieldset_helper import TEMPLATE_NAME, get_context

register = template.Library()


@register.simple_tag(takes_context=True)
def fieldset(context, field):
    '''
    This template tag makes it easy to render form fields as
    <fieldset> elements.
    '''

    t = context.template.engine.get_template(TEMPLATE_NAME)

    return t.render(template.Context(get_context(field)))



from django.test import TestCase


class ViewTests(TestCase):
    def test_home_returns_200(self):
        res = self.client.get('/')
        self.assertEqual(res.status_code, 200)

    def test_nonexistent_example_returns_404(self):
        res = self.client.get('/example/zzz')
        self.assertContains(res, 'Example not found', status_code=404)



from jinja2 import Environment
import uswds_forms


def environment(**options):
    env = Environment(**options)
    env.globals.update({
        'fieldset': uswds_forms.fieldset,
    })
    return env



import re
from pathlib import Path
from typing import Match
from django.conf import settings
from django.utils.safestring import SafeString
from django.utils.module_loading import import_string
from django.urls import reverse

from .render_source import render_template_source, render_python_source

APP_DIR = Path(__file__).resolve().parent
PY_DIR = APP_DIR / 'examples'
TEMPLATES_DIR = APP_DIR / 'templates' / 'examples'
JINJA2_DIR = APP_DIR / 'jinja2' / 'examples'


def add_links_to_docs(text: str) -> str:
    '''
    This somewhat hacky function takes a string of documentation and
    adds Sphinx documentation hyperlinks to anything that looks like a
    reference to a uswds_forms object.

    Note that it's assumed the text passed in is trusted.
    '''

    def hyperlink(match: Match) -> str:
        end_text = ''
        objname = match.group(0)
        if objname.endswith('.'):
            end_text = objname[-1]
            objname = objname[:-1]
        _, short_objname = objname.split('.', 1)

        # Try importing the string, to make sure it's not pointing at
        # a symbol that doesn't actually exist.
        import_string(objname)

        return '<a href="{}reference.html#{}"><code>{}</code></a>{}'.format(
            settings.DOCS_URL,
            objname,
            short_objname,
            end_text
        )

    text = re.sub(
        r'uswds_forms\.([a-zA-Z0-9_.]+)',
        hyperlink,
        text
    )
    return text


class Example:
    def __init__(self, basename: str, load: bool=False) -> None:
        self.basename = basename
        self.template_path = TEMPLATES_DIR / (basename + '.html')
        self.jinja2_path = JINJA2_DIR / (basename + '.html')
        self.python_path = PY_DIR / (basename + '.py')

        if load:
            self.load()

    def load(self):
        self.view = import_string('app.examples.' + self.basename + '.view')
        self.module = import_string('app.examples.' + self.basename)

        docstr = import_string('app.examples.' + self.basename + '.__doc__')
        self.name, description = docstr.split('\n\n', 1)
        self.description = SafeString(add_links_to_docs(description))
        self.python_source = render_python_source(self.python_path)

    @property
    def template_source(self):
        return render_template_source(self.template_path)

    @property
    def jinja2_source(self):
        return render_template_source(self.jinja2_path)

    @property
    def url(self):
        return reverse('example', args=(self.basename,))

    def render(self, request):
        # This is sort of weird because we're decoding the
        # content in a HttpResponse; it's not an ideal API,
        # but we want the examples be as conventional as
        # possible so they don't confuse readers, and having
        # them return HttpResponse objects like normal Django
        # views is the easiest way to accomplish that.
        return SafeString(self.view(request).content.decode('utf-8'))



import os
from setuptools import setup, find_packages
import distutils.cmd
import subprocess

import metadata


class SimpleCommand(distutils.cmd.Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass


class DevDocsCommand(SimpleCommand):
    description = "Run development server for documentation"

    def run(self):
        subprocess.check_call(
            ['sphinx-autobuild', '.', '_build/html', '-p', '8001',
             '-z', os.path.join('..', 'uswds_forms')],
            cwd='docs'
        )


setup(name='django-uswds-forms',
      cmdclass={
          'devdocs': DevDocsCommand,
      },
      zip_safe=False,
      version=metadata.get_version(),
      description='Django Forms integration with the U.S. Web Design Standards',

      # TODO: Add long_description.

      author='Atul Varma',
      author_email='atul.varma@gsa.gov',
      license='Public Domain',
      url='https://github.com/18F/django-uswds-forms',
      package_dir={'uswds_forms': 'uswds_forms'},
      include_package_data=True,
      packages=find_packages(),
      install_requires=[
          'django>=1.11.1,<2',
      ],

      classifiers=[
          'Development Status :: 3 - Alpha',
          'Environment :: Web Environment',
          'Framework :: Django',
          'Framework :: Django :: 1.11',
          'Intended Audience :: Developers',
          'License :: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Topic :: Software Development :: Libraries :: Python Modules',
          'Programming Language :: Python',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Topic :: Utilities'],
      )



from django.test import TestCase, override_settings

from ..example import add_links_to_docs


@override_settings(DOCS_URL="http://docs/")
class AddLinksToDocsTests(TestCase):
    def test_it_does_nothing_when_docs_have_no_links(self):
        self.assertEqual(add_links_to_docs('foo bar baz'), 'foo bar baz')

    def test_it_hyperlinks_uswds_links(self):
        self.assertEqual(
            add_links_to_docs('foo uswds_forms.UswdsForm baz'),
            'foo <a href="http://docs/reference.html#uswds_forms.UswdsForm">'
            '<code>UswdsForm</code></a> baz'
        )

    def test_it_does_not_hyperlink_final_periods(self):
        self.assertEqual(
            add_links_to_docs('foo uswds_forms.UswdsForm. yup.'),
            'foo <a href="http://docs/reference.html#uswds_forms.UswdsForm">'
            '<code>UswdsForm</code></a>. yup.'
        )



from pathlib import Path
from typing import Dict  # NOQA

MY_DIR = Path(__file__).resolve().parent

INIT_PY = MY_DIR / 'uswds_forms' / '__init__.py'


def get_version() -> str:
    '''
    Get the version number. It's in our package's __init__.py, but
    we don't want to import it directly b/c this will bring in
    a bunch of other dependencies that might not be on the user's
    system, so we need to be a bit hacky here.
    '''

    for line in open(str(INIT_PY)):
        if line.startswith('VERSION'):
            globs = {}  # type: Dict[str, str]
            exec(line, globs)
            return globs['VERSION']

    raise Exception('VERSION not found!')



from django.forms.utils import ErrorList
from django.utils.html import format_html, format_html_join

__all__ = (
    'UswdsErrorList',
)


class UswdsErrorList(ErrorList):
    '''
    This is an error list formatter that renders errors in the style used
    by USWDS forms. For an example of how this makes errors look in
    practice, see the `USWDS text inputs example
    <https://standards.usa.gov/components/form-controls/#text-input>`_.

    Note that you probably don't need to use this class directly, as
    :class:`~uswds_forms.UswdsForm` uses it by default.

    For more details on how to use this class, see the Django
    documentation on `customizing the error list format <https://docs.djangoproject.com/en/1.11/ref/forms/api/#customizing-the-error-list-format>`_.
    '''

    def as_ul(self):
        # Note that because we're a subclass of list, we're basically
        # just testing to see if we're an empty list here.
        if not self:
            return ''

        return format_html(
            '<ul class="usa-unstyled-list">{}</ul>',
            format_html_join(
                '',
                '<li class="usa-input-error-message" role="alert">{}</li>',
                ((e,) for e in self)
            )
        )
